#ifndef _SQUID_DAEMON_H
#define _SQUID_DAEMON_H

#include <time.h>
#include <vector>
#include <string>
#include <iostream>
#include <fstream>

#include "myldap.h"
#include "mysqlsave.h"
#include "mydata.h"

#define  MAX_PKT_SIZE     0xffff
#define  numListenThread  20
#define  numLocalService  40
#define  numAuthService		20

using std::string;
using std::vector;
using std::iterator;

			char daemonDir[1024], baseDir[1024], logDir[1024], logFileName[1024], pidFileName[1024], ldapStr[1024];
			unsigned short listenPort, servicePort, authPort, proxyPort, clientPort;
			struct in_addr daemonServerIP;
			char daemonServer[18];//ldapServer[256]
			char effective_user[256], effective_group[256];
			time_t allowTime, close_time, ldapTimeUpdate;

class SquidDaemon
{
	protected:
		char myHostName[256], myIPstr[18];

		int parentPid, childPid;

		
		bool createSocket(int &sock, u_int32_t s_addr, int s_port);
		int alreadyRun();
		void daemonize();
		bool pripareNet();
		void startChild();
		static void stopParentDaemon(int signum);
		//vectorAttrs locVectorAttrs, locBaseInfo;
		ServerInfo loc_si;
public:
		volatile bool workDaemon;
		int sockfd, sockRL, sockAP;

		void execute();
		SquidDaemon(ServerInfo si);
		//SquidDaemon(vectorAttrs myVectorAttrs, vectorAttrs myBaseInfo);
		~SquidDaemon();
};

class SquidChild
{
	protected:
	
		vector <SUserData> collected;
		vector <SIdentUser> share;
		//myLDAP lsConn[numListenThread];
		char myHostName[256], myIPstr[18];
		int sockfd, sockRL, sockAP;
		
		ServerInfo child_si;
		mySqlSave sqlbase;

		bool clientAccess(int fd, struct in_addr userIPaddr, char *userIPstr);

		int registerUserIP(struct in_addr userIPaddr, char *userIPstr, char *userName, int &getLimit, uint &uId);
		void deregisterUserIP(struct in_addr userIPaddr, char *userIPstr);
		void sendToSquid(int fd, struct in_addr rlIPaddr, char *rlIPstr,  uint &cUserID);
		void sendToAuth(int fd, struct in_addr rlIPaddr, char *rlIPstr);
		
		void saveUserTrafficToDisk(SUserData &sud);
		void saveTraffic();

		bool sendRequest(struct in_addr userIPaddr, char *userIPstr, uint &uId);
		bool userInfo(uint &i, uint &cUserID, char *cached, int myCacheAdd, time_t realTime);

		static void stopChildDaemon(int signum);

		static void *listenerThread(void *data);
		static void *serviceThread(void *data);
		static void *authThread(void *data);

		/*static void *answerListenerThread(void *data);
		static void *answerServiceThread(void *data);
		static void *answerAuthThread(void *data);*/		
		int checkUserLimit(int userID);
		//vectorAttrs locVectorAttrs;

	public:
		pthread_mutex_t request_mutex;
		pthread_mutex_t sql_mutex;
		myLDAP lsConn;

		volatile bool workChildDaemon, readyChildToWork;

		void childDaemonJob();

		pthread_t listen_thread[numListenThread], service_thread[numLocalService], auth_thread[numAuthService];

		//SquidChild(int _sockfd, int _sockRL, int _sockAP, char *_myHostName, char *_myIPstr, vectorAttrs myVectorAttrs, vectorAttrs myBaseInfo);
		SquidChild(int _sockfd, int _sockRL, int _sockAP, char *_myHostName, char *_myIPstr, ServerInfo si);
		~SquidChild();
};

/*struct	SAnswerData
{
	SquidChild *pToChD;
	int fd;
	struct in_addr userIPaddr;
	char userIPstr[18];
};*/

bool myCrypt(char *strCript);
void print_to_file (FILE *fp, const char *format, va_list args);
void saveLog(bool ntime, const char *format,...);
void stringToLow(char *inStr);

#endif /*_SERV_DAEMON_H*/
