/*
//ͻ
// (c) 2003 Novell, Inc.  All rights reserved.                              
//                                                                          
// THIS WORK IS SUBJECT TO U.S. AND INTERNATIONAL COPYRIGHT LAWS AND        
// TREATIES.                                                                
//                                                                          
// NO PART OF THIS WORK MAY BE USED, PRACTICED, PERFORMED, COPIED,          
// DISTRIBUTED, REVISED, MODIFIED, TRANSLATED, ABRIDGED, CONDENSED,         
// EXPANDED, COLLECTED, COMPILED, LINKED, RECAST, TRANSFORMED OR ADAPTED    
// WITHOUT THE PRIOR WRITTEN CONSENT OF NOVELL, INC.                        
//                                                                          
// ANY USE OR EXPLOITATION OF THIS WORK WITHOUT AUTHORIZATION COULD SUBJECT 
// THE PERPETRATOR TO CRIMINAL AND CIVIL LIABILITY.                         
//͹
// Source module name: NWDP_ATT.H                                           
//͹
// Library VeRsIoN=1.8.6                                                    
//ͼ
*/
/* Inclusion Control */
#if !defined( NWDP_ATT_H )
   #define NWDP_ATT_H

/* Standard Library Header Includes */

   #if !defined(NTYPES_H)
      #define N_INC_NO_OLD_CONSTANTS
      #define N_INC_NO_OLD_MACROS
      #include <ntypes.h>
      #undef N_INC_NO_OLD_MACROS
      #undef N_INC_NO_OLD_CONSTANTS
   #endif /* NTYPES_H */


   #if !defined(NWDP_TXT_H)
      #include <nwdp_txt.h>
   #endif /* NWDP_TXT_H */

/* Header Includes */

/* Manifest Constant Definitions */

#define NWDP_EVENT_TYPE_VALUE_CHANGE    0
#define NWDP_EVENT_TYPE_ERROR           1
#define NWDP_EVENT_TYPE_WARNING         2
#define NWDP_EVENT_TYPE_REPORT          3
#define NWDP_HOLD_UNTIL_NO_HOLD          0
#define NWDP_HOLD_UNTIL_DAYTIME          1
#define NWDP_HOLD_UNTIL_EVENING          2
#define NWDP_HOLD_UNTIL_NIGHT            3
#define NWDP_HOLD_UNTIL_WEEKEND          4
#define NWDP_HOLD_UNTIL_SECOND_SHIFT     5
#define NWDP_HOLD_UNTIL_THIRD_SHIFT      6
#define NWDP_HOLD_UNTIL_INDEFINITE       7


   /*
   // The 'oidLength' field of the NWDPOid structure cannot be more
   // than NWDP_OID_MAX_LENGTH.
   */
   #define NWDP_OID_MAX_LENGTH          127

   #if !defined(NWDP_NSRV_MAX_DN_CHARS)
      #define NWDP_NSRV_MAX_DN_CHARS          256
      #define NWDP_NSRV_MAX_DN_BYTES \
            ((NWDP_NSRV_MAX_DN_CHARS+1)*2)

      #define NWDP_NSRV_MAX_TREE_CHARS        32
      #define NWDP_NSRV_MAX_TREE_BYTES \
            ((NWDP_NSRV_MAX_TREE_CHARS+1)*2)

      #define NWDP_NSRV_GUID_BYTES            16
   #endif


/* Macro Definitions */

   #define NWDPOidLenMac(__oidPtr)                 \
      ((((pNWDPOid)(__oidPtr))->oidLength)      + \
       sizeof(((pNWDPOid)(__oidPtr))->asn1Type) + \
       sizeof(((pNWDPOid)(__oidPtr))->oidLength) )


/* Type Definitions */

#define N_PACK_1
#include <npackon.h>


   typedef struct
   {
      nuint8 asn1Type;                    /* OID is a type 6 */
      nuint8 oidLength;                   /* excludes this byte; count of
                                             significant OID digits.*/
      nuint8 oid[NWDP_OID_MAX_LENGTH];    /* octet str; when stored in files
                                             it is zero filled.  */
   } NWDPOid,N_FAR *pNWDPOid;

   /*
   // The Fully Qualified Name or FQN for NDS consists of the 'dnPtr' field
   // which points to the NetWare 4.x Distinguished Name string component
   // and the optional Tree Name.  If the 'treeOptionPtr' field is NULL then
   // the API which expects the NWDPNSrvFQN will assume the NDS Context
   // has had the DCK_TREE_NAME buffer set previously to correspond.
   */
   typedef enum {
      NWDP_NSRV_SYNTAX_NDS_WITH_TREE = 0
   } NWDPNSrvSyntaxEnum, N_FAR *pNWDPNSrvSyntaxEnum;

   typedef struct {
      NWDPNSrvSyntaxEnum designator;
      union {
       struct {
          pnstr16  dn16Ptr;
          pnstr16  treeOption16Ptr;
       } ndsWithTree;
      } u;
   } NWDPNSrvFQN, N_FAR *pNWDPNSrvFQN;

   /*
   // The NWDPNSrvFQNBuffer is intended to be capable of using
   // the least amount of memory. This means that the strings
   // referred to by the 'fqn' structure's pointers are packed into the 
   // ndsWithTree buffer array and any tree name will appear at
   // ndsWithTree[unilen(&ndsWithTree[0])+1].
   */
   typedef struct {
      NWDPNSrvFQN fqn;
      union {
       nchar16 ndsWithTree[NWDP_NSRV_MAX_DN_CHARS +
                           NWDP_NSRV_MAX_TREE_CHARS + 2];
      } bufferData;
   } NWDPNSrvFQNBuffer, N_FAR *pNWDPNSrvFQNBuffer;


typedef enum {
	NWDP_AVT_NULL = 0,
	NWDP_AVT_TEXT = 1,
	NWDP_AVT_DESCRIPTIVE_NAME = 2,
	NWDP_AVT_DESCRIPTOR = 3,
	NWDP_AVT_MESSAGE = 4,
	NWDP_AVT_ERROR_MESSAGE = 5,
	NWDP_AVT_SIMPLE_NAME = 6,
	NWDP_AVT_DIST_NAME_STR = 7,
	NWDP_AVT_DIST_NAME_STR_SEQ = 8,
	NWDP_AVT_DELTA_TIME = 9,
	NWDP_AVT_TIME = 10,
	NWDP_AVT_INTEGER = 11,
	NWDP_AVT_INTEGER_SEQ = 12,
	NWDP_AVT_CARDINAL = 13,
	NWDP_AVT_CARDINAL_SEQ = 14,
	NWDP_AVT_POSITIVE_INTEGER = 15,
	NWDP_AVT_INTEGER_RANGE = 16,
	NWDP_AVT_CARDINAL_RANGE = 17,
	NWDP_AVT_MAXIMUM_INTEGER = 18,
	NWDP_AVT_MINIMUM_INTEGER = 19,
	NWDP_AVT_INTEGER_64 = 20,
	NWDP_AVT_INTEGER_64_SEQ = 21,
	NWDP_AVT_CARDINAL_64 = 22,
	NWDP_AVT_CARDINAL_64_SEQ = 23,
	NWDP_AVT_POSITIVE_INTEGER_64 = 24,
	NWDP_AVT_INTEGER_64_RANGE = 25,
	NWDP_AVT_CARDINAL_64_RANGE = 26,
	NWDP_AVT_MAXIMUM_INTEGER_64 = 27,
	NWDP_AVT_MINIMUM_INTEGER_64 = 28,
	NWDP_AVT_REAL = 29,
	NWDP_AVT_REAL_SEQ = 30,
	NWDP_AVT_NON_NEGATIVE_REAL = 31,
	NWDP_AVT_REAL_RANGE = 32,
	NWDP_AVT_NON_NEGATIV_REAL_RANGE = 33,
	NWDP_AVT_BOOLEAN = 34,
	NWDP_AVT_PERCENT = 35,
	NWDP_AVT_OBJECT_IDENTIFIER = 36,
	NWDP_AVT_OBJECT_IDENTIFIER_SEQ = 37,
	NWDP_AVT_NAME_OR_OID = 38,
	NWDP_AVT_NAME_OR_OID_SEQ = 39,
	NWDP_AVT_DISTINGUISHED_NAME = 40,
	NWDP_AVT_RDN_SEQUENCE = 41,
	NWDP_AVT_REALIZATION = 42,
	NWDP_AVT_MEDIUM_DIMENSIONS = 43,
	NWDP_AVT_DIMENSION = 44,
	NWDP_AVT_XY_DIMENSIONS = 45,
	NWDP_AVT_LOCATIONS = 46,
	NWDP_AVT_AREA = 47,
	NWDP_AVT_AREA_SEQ = 48,
	NWDP_AVT_EDGE = 49,
	NWDP_AVT_FONT_REFERENCE = 50,
	NWDP_AVT_CARDINAL_OR_OID = 51,
	NWDP_AVT_OID_CARDINAL_MAP = 52,
	NWDP_AVT_CARDINAL_OR_NAM_OR_OID = 53,
	NWDP_AVT_POSITIVE_INT_OR_OID = 54,
	NWDP_AVT_EVENT_HANDLING_PROFILE = 55,
	NWDP_AVT_OCTET_STRING = 56,
	NWDP_AVT_PRIORITY = 57,
	NWDP_AVT_LOCALE = 58,
	NWDP_AVT_METHOD_DELIVERY_ADDR = 59,
	NWDP_AVT_OBJECT_IDENTIFICATION = 60,
	NWDP_AVT_RESULTS_PROFILE = 61,
	NWDP_AVT_CRITERIA = 62,
	NWDP_AVT_JOB_PASSWORD = 63,
	NWDP_AVT_JOB_LEVEL = 64,
	NWDP_AVT_JOB_CATEGORIES = 65,
	NWDP_AVT_PRINT_CHECKPOINT = 66,
	NWDP_AVT_IGNORED_ATTRIBUTE = 67,
	NWDP_AVT_RESOURCE = 68,
	NWDP_AVT_MEDIUM_SUBSTITUTION = 69,
	NWDP_AVT_FONT_SUBSTITUTION = 70,
	NWDP_AVT_RESOURCE_CONTEXT_SEQ = 71,
	NWDP_AVT_SIDES = 72,
	NWDP_AVT_PAGE_SELECT_SEQ = 73,
	NWDP_AVT_PAGE_MEDIA_SELECT = 74,
	NWDP_AVT_DOCUMENT_CONTENT = 75,
	NWDP_AVT_PAGE_SIZE = 76,
	NWDP_AVT_PRESENTATION_DIRECTION = 77,
	NWDP_AVT_PAGE_ORDER = 78,
	NWDP_AVT_FILE_REFERENCE = 79,
	NWDP_AVT_MEDIUM_SOURCE_SIZE = 80,
	NWDP_AVT_INPUT_TRAY_MEDIUM = 81,
	NWDP_AVT_OUTPUT_BINS_CHARS = 82,
	NWDP_AVT_PAGE_ID_TYPE = 83,
	NWDP_AVT_LEVEL_RANGE = 84,
	NWDP_AVT_CATEGORY_SET = 85,
	NWDP_AVT_NUMBERS_UP_SUPPORTED = 86,
	NWDP_AVT_FINISHING = 87,
	NWDP_AVT_PRT_CONTAINED_OBJ_ID = 88,
	NWDP_AVT_PRT_CONFIG_OBJECT_ID = 89,
	NWDP_AVT_TYPED_NAME = 90,
	NWDP_AVT_NET_ADDRESS = 91,
	NWDP_AVT_XY_DIMENSIONS_VALUE = 92,
	NWDP_AVT_NAME_OR_OID_DIM_MAP = 93,
	NWDP_AVT_PRINTER_STATE_REASON = 94,
	NWDP_AVT_ENUMERATION = 95,
	NWDP_AVT_QUALIFIED_NAME = 96,
	NWDP_AVT_QUALIFIED_NAME_SET = 97,
	NWDP_AVT_COLORANT_SET = 98,
	NWDP_AVT_RESOURCE_PRINTER_ID = 99,
	NWDP_AVT_EVENT_OBJECT_ID = 100,
	NWDP_AVT_QUALIFIED_NAME_MAP = 101,
	NWDP_AVT_FILE_PATH = 102,
	NWDP_AVT_URI = 103,
	NWDP_AVT_CARDNL_OR_ENUM_OR_TIME = 104,
	NWDP_AVT_PRT_CONTAIN_OBJ_ID_SET = 105,
	NWDP_AVT_OCTET_STRING_PAIR = 106,
	NWDP_AVT_OCTET_STRING_INT_PAIR = 107,
	NWDP_AVT_XRI = 108,
	NWDP_AVT_EVENT_HANDLING_PROFIL2 = 109
} NWDPAVTEnum, N_FAR *pNWDPAVTEnum;

typedef struct {
	nuint itemCount;
	pnint8 itemPtr;
} NWDPOctetString, N_FAR *pNWDPOctetString;

typedef struct {
	nint32 flag;
	nint32 value;
} NWDPIntegerOption, N_FAR *pNWDPIntegerOption;

typedef struct {
	nuint oidStructSize;
	pNWDPOid oidStructPtr;
} NWDPObjectIdentifier, N_FAR *pNWDPObjectIdentifier;


typedef enum {
	NWDP_NAME_OR_OID_NONE = 0,
	NWDP_NAME_OR_OID_GLOBAL = 1,
	NWDP_NAME_OR_OID_LOCAL = 2
} NWDPNameOrOidEnum, N_FAR *pNWDPNameOrOidEnum;

typedef struct {
	NWDPNameOrOidEnum designator;
	union {
		NWDPObjectIdentifier globalForm;
		NWDPText localForm;
	} u;
} NWDPNameOrOid, N_FAR *pNWDPNameOrOid;

typedef struct {
	NWDPText name;
	pNWDPNameOrOid syntaxOptionPtr;
} NWDPDistinguishedNameString, N_FAR *pNWDPDistinguishedNameString;

typedef enum {
	NWDP_QUALIFIED_NAME_NONE = 0,
	NWDP_QUALIFIED_NAME_SIMPLE = 1,
	NWDP_QUALIFIED_NAME_NDS = 2
} NWDPQualifiedNameEnum, N_FAR *pNWDPQualifiedNameEnum;

typedef struct {
	NWDPText object;
	NWDPText treeOption;
} NWDPNdsName, N_FAR *pNWDPNdsName;

typedef struct {
	NWDPQualifiedNameEnum designator;
	union {
		NWDPText simpleName;
		NWDPNdsName ndsName;
	} u;
} NWDPQualifiedName, N_FAR *pNWDPQualifiedName;

typedef struct {
	nuint itemCount;
	pNWDPQualifiedName itemPtr;
} NWDPQualifiedNameSet, N_FAR *pNWDPQualifiedNameSet;

typedef struct {
	nuint itemCount;
	pNWDPDistinguishedNameString itemPtr;
} NWDPDistinguishedNameStrSeq, N_FAR *pNWDPDistinguishedNameStrSeq;

typedef struct {
	nuint itemCount;
	pnint32 itemPtr;
} NWDPIntegerSeq, N_FAR *pNWDPIntegerSeq;

typedef struct {
	nuint itemCount;
	pnuint32 itemPtr;
} NWDPCardinalSeq, N_FAR *pNWDPCardinalSeq;

typedef struct {
	nint32 lowerBound;
	nint32 upperBound;
} NWDPIntegerRange, N_FAR *pNWDPIntegerRange;

typedef struct {
	nuint32 lowerBound;
	nuint32 upperBound;
} NWDPCardinalRange, N_FAR *pNWDPCardinalRange;

typedef struct {
	nint32 flag;
	nreal64 value;
} NWDPRealOption, N_FAR *pNWDPRealOption;

typedef struct {
	nuint itemCount;
	pnreal64 itemPtr;
} NWDPRealSet, N_FAR *pNWDPRealSet;

typedef struct {
	nreal64 lowerBound;
	nreal64 upperBound;
} NWDPRealRange, N_FAR *pNWDPRealRange;

typedef struct {
	nuint itemCount;
	pNWDPObjectIdentifier itemPtr;
} NWDPObjectIdentifierSet, N_FAR *pNWDPObjectIdentifierSet;

typedef struct {
	nuint itemCount;
	pNWDPNameOrOid itemPtr;
} NWDPNameOrOidSet, N_FAR *pNWDPNameOrOidSet;

typedef struct {
	nuint itemCount;
	pNWDPText itemPtr;
} NWDPRDNSequence, N_FAR *pNWDPRDNSequence;

typedef enum {
	NWDP_REALIZATION_LOGICAL = 0,
	NWDP_REALIZATION_PHYSICAL = 1,
	NWDP_REALIZATION_LOG_AND_PHYS = 2
} NWDPRealizationEnum, N_FAR *pNWDPRealizationEnum;

typedef struct {
	nreal64 xDimension;
	nreal64 yDimension;
} NWDPXYRealDimensions, N_FAR *pNWDPXYRealDimensions;

typedef enum {
	NWDP_DIMENSION_NUMERIC = 0,
	NWDP_DIMENSION_NAMED = 1
} NWDPDimValueEnum, N_FAR *pNWDPDimValueEnum;

typedef struct {
	NWDPDimValueEnum designator;
	union {
		nreal64 numericValue;
		NWDPNameOrOid namedValue;
	} u;
} NWDPDimValue, N_FAR *pNWDPDimValue;

typedef struct {
	NWDPDimValue value;
	NWDPRealOption toleranceOption;
} NWDPDimension, N_FAR *pNWDPDimension;

typedef struct {
	nuint32 xDimension;
	nuint32 yDimension;
} NWDPXYCardinalDimensions, N_FAR *pNWDPXYCardinalDimensions;

typedef enum {
	NWDP_DIM_XY_REAL = 0,
	NWDP_DIM_XY_NAMED = 1,
	NWDP_DIM_XY_CARDINAL = 2
} NWDPXYDimensionsValueEnum, N_FAR *pNWDPXYDimensionsValueEnum;

typedef struct {
	NWDPXYDimensionsValueEnum designator;
	union {
		NWDPXYRealDimensions realValue;
		NWDPNameOrOid namedValue;
		NWDPXYCardinalDimensions cardinalValue;
	} u;
} NWDPXYDimensionsValue, N_FAR *pNWDPXYDimensionsValue;

typedef struct {
	NWDPXYDimensionsValue value;
	NWDPRealOption toleranceOption;
} NWDPXYDimensions, N_FAR *pNWDPXYDimensions;

typedef enum {
	NWDP_LOCATION_NUMERIC = 0,
	NWDP_LOCATION_NAMED = 1
} NWDPLocationValueEnum, N_FAR *pNWDPLocationValueEnum;

typedef struct {
	NWDPLocationValueEnum designator;
	union {
		NWDPRealSet numericValueSeq;
		NWDPNameOrOid namedValue;
	} u;
} NWDPLocationValue, N_FAR *pNWDPLocationValue;

typedef struct {
	NWDPLocationValue value;
	NWDPRealOption toleranceOption;
} NWDPLocations, N_FAR *pNWDPLocations;

typedef struct {
	nreal64 minimumX;
	nreal64 maximumX;
	nreal64 minimumY;
	nreal64 maximumY;
} NWDPArea, N_FAR *pNWDPArea;

typedef struct {
	nuint itemCount;
	pNWDPArea itemPtr;
} NWDPAreaSeq, N_FAR *pNWDPAreaSeq;

typedef enum {
	NWDP_EDGE_BOTTOM = 0,
	NWDP_EDGE_RIGHT = 1,
	NWDP_EDGE_TOP = 2,
	NWDP_EDGE_LEFT = 3
} NWDPEdge, N_FAR *pNWDPEdge;

typedef struct {
	nint32 flag;
	NWDPEdge value;
} NWDPEdgeOption, N_FAR *pNWDPEdgeOption;

typedef enum {
	NWDP_CARDINAL_OR_OID_NUMBER = 0,
	NWDP_CARDINAL_OR_OID_ID = 1
} NWDPCardinalOrOidEnum, N_FAR *pNWDPCardinalOrOidEnum;

typedef struct {
	NWDPCardinalOrOidEnum designator;
	union {
		nint32 cardinal;
		NWDPObjectIdentifier oid;
	} u;
} NWDPCardinalOrOid, N_FAR *pNWDPCardinalOrOid;

typedef struct {
	NWDPObjectIdentifier oidOption;
	nuint32 cardinal;
} NWDPOidCardinalMap, N_FAR *pNWDPOidCardinalMap;

typedef enum {
	NWDP_CARDINAL_OR_NAM_OR_OID_NUM = 0,
	NWDP_CARDINAL_OR_NAME_OR_OID_ID = 1
} NWDPCardinalOrNameOrOidEnum, N_FAR *pNWDPCardinalOrNameOrOidEnum;

typedef struct {
	NWDPCardinalOrNameOrOidEnum designator;
	union {
		nint32 cardinal;
		NWDPNameOrOid nameOrOid;
	} u;
} NWDPCardinalOrNameOrOid, N_FAR *pNWDPCardinalOrNameOrOid;

typedef enum {
	NWDP_INT_OR_OID_ID = 0,
	NWDP_INT_OR_OID_NUMBER = 1
} NWDPPositivIntegerOrOidEnum, N_FAR *pNWDPPositivIntegerOrOidEnum;

typedef struct {
	NWDPPositivIntegerOrOidEnum designator;
	union {
		NWDPObjectIdentifier oid;
		nint32 integer;
	} u;
} NWDPPositiveIntegerOrOid, N_FAR *pNWDPPositiveIntegerOrOid;

typedef struct {
	NWDPText printerName;
	nuint32 localIdentifier;
} NWDPPrtContainedObjectId, N_FAR *pNWDPPrtContainedObjectId;

typedef struct {
	nuint itemCount;
	pNWDPPrtContainedObjectId itemPtr;
} NWDPPrtContainedObjectIdSet, N_FAR *pNWDPPrtContainedObjectIdSet;

typedef struct {
	NWDPPrtContainedObjectId jobIdentifier;
	nuint32 documentNumber;
} NWDPDocumentIdentifier, N_FAR *pNWDPDocumentIdentifier;

typedef enum {
	NWDP_DA_OR_ADDR_AND_OR_DIR_NAME = 0,
	NWDP_DA_DISTINGUISHED_NAME = 1,
	NWDP_DA_TEXT = 2,
	NWDP_DA_OCTET_STRING = 3,
	NWDP_DA_DIST_NAME_STRING = 4,
	NWDP_DA_RPC_ADDRESS = 5,
	NWDP_DA_QUALIFIED_NAME = 6
} NWDPDeliveryAddressEnum, N_FAR *pNWDPDeliveryAddressEnum;

typedef struct {
	NWDPDeliveryAddressEnum designator;
	union {
		NWDPText mhsAddress;
		NWDPText distinguishedName;
		NWDPText text;
		NWDPOctetString octetString;
		NWDPDistinguishedNameString distinguishedNameString;
		NWDPQualifiedName qualifiedName;
	} u;
} NWDPDeliveryAddress, N_FAR *pNWDPDeliveryAddress;

typedef struct {
	NWDPText printerName;
	NWDPQualifiedName qualifiedName;
} NWDPPrtConfigObjectId, N_FAR *pNWDPPrtConfigObjectId;

typedef struct {
	NWDPText name;
	NWDPObjectIdentifier containingClassOid;
	nint32 eventType;
} NWDPEventObjectId, N_FAR *pNWDPEventObjectId;

typedef struct {
	NWDPQualifiedName primary;
	NWDPQualifiedName secondary;
} NWDPQualifiedNameMap, N_FAR *pNWDPQualifiedNameMap;

typedef enum {
	NWDP_CARD_OR_ENUM_OR_TIME_CARD = 0,
	NWDP_CARD_OR_ENUM_OR_TIME_ENUM = 1,
	NWDP_CARD_OR_ENUM_OR_TIME_TIME = 2
} NWDPCardinalOrEnumOrTimeEnum, N_FAR *pNWDPCardinalOrEnumOrTimeEnum;

typedef struct {
	NWDPCardinalOrEnumOrTimeEnum designator;
	union {
		nuint32 cardinal;
		nint32 enumeration;
		nuint32 time;
	} u;
} NWDPCardinalOrEnumOrTime, N_FAR *pNWDPCardinalOrEnumOrTime;

typedef struct {
	NWDPOctetString octetString1;
	NWDPOctetString octetString2;
} NWDPOctetStringPair, N_FAR *pNWDPOctetStringPair;

typedef struct {
	NWDPOctetString octetString;
	nint32 integer1;
	nint32 integer2;
} NWDPOctetStringIntegerPair, N_FAR *pNWDPOctetStringIntegerPair;

typedef struct {
	nuint itemCount;
	pnint8 itemPtr;
} NWDPPrivAttrCertificate, N_FAR *pNWDPPrivAttrCertificate;

typedef struct {
	NWDPPrivAttrCertificate operationPacOption;
	NWDPPrivAttrCertificate proxyPacOption;
} NWDPPrivileges, N_FAR *pNWDPPrivileges;

typedef struct {
	NWDPObjectIdentifier method;
	NWDPDeliveryAddress address;
} NWDPDeliveryAddrForMethod, N_FAR *pNWDPDeliveryAddrForMethod;

typedef enum {
	NWDP_OBJ_ID_NONE = 0,
	NWDP_OBJ_ID_PRT_CONTAIND_OBJ_ID = 1,
	NWDP_OBJ_ID_DOCUMENT_IDENTIFIER = 2,
	NWDP_OBJ_ID_OBJECT_IDENTIFIER = 3,
	NWDP_OBJ_ID_OBJECT_NAME = 4,
	NWDP_OBJ_ID_NAME_OR_OID = 5,
	NWDP_OBJ_ID_SIMPLE_NAME = 6,
	NWDP_OBJ_ID_PRT_CONFIG_OBJ_ID = 7,
	NWDP_OBJ_ID_QUALIFIED_NAME = 8,
	NWDP_OBJ_ID_EVENT_OBJECT_ID = 9
} NWDPObjIdentificationEnum, N_FAR *pNWDPObjIdentificationEnum;

typedef struct {
	NWDPObjIdentificationEnum designator;
	union {
		NWDPPrtContainedObjectId prtContainedObjectId;
		NWDPDocumentIdentifier documentIdentifier;
		NWDPObjectIdentifier objectIdentifier;
		NWDPDistinguishedNameString objectName;
		NWDPNameOrOid nameOrOid;
		NWDPText simpleName;
		NWDPPrtConfigObjectId prtConfigObjectId;
		NWDPQualifiedName qualifiedName;
		NWDPEventObjectId eventObjectId;
	} u;
} NWDPObjectIdentification, N_FAR *pNWDPObjectIdentification;

typedef struct {
	nuint itemCount;
	pNWDPObjectIdentification itemPtr;
} NWDPObjectIdentificationSeq, N_FAR *pNWDPObjectIdentificationSeq;

typedef struct {
	NWDPText name;
	nint32 level;
	nint32 interval;
} NWDPTypedName, N_FAR *pNWDPTypedName;

typedef enum {
	NWDP_NET_IPX = 0,
	NWDP_NET_IP = 1,
	NWDP_NET_SDLC = 2,
	NWDP_NET_TOKENRING_ETHERNET = 3,
	NWDP_NET_OSI = 4,
	NWDP_NET_APPLETALK = 5,
	NWDP_NET_COUNT = 6
} NWDPNetAddressTypeEnum, N_FAR *pNWDPNetAddressTypeEnum;

typedef struct {
	NWDPNetAddressTypeEnum type;
	NWDPOctetString address;
} NWDPNetAddress, N_FAR *pNWDPNetAddress;

typedef struct {
	NWDPNameOrOid nameOrOid;
	NWDPXYCardinalDimensions value;
} NWDPNameOrOidDimensionMap, N_FAR *pNWDPNameOrOidDimensionMap;

typedef enum {
	NWDP_STATE_SEVERITY_OTHER = 1,
	NWDP_STATE_SEVERITY_WARNING = 1 + 1,
	NWDP_STATE_SEVERITY_CRITICAL = 1 + 2
} NWDPStateSeverityEnum, N_FAR *pNWDPStateSeverityEnum;

typedef enum {
	NWDP_TRAINING_OTHER = 1,
	NWDP_TRAINING_UNKNOWN = 1 + 1,
	NWDP_TRAINING_UNTRAINED = 1 + 2,
	NWDP_TRAINING_TRAINED = 1 + 3,
	NWDP_TRAINING_FIELD_SERVICE = 1 + 4,
	NWDP_TRAINING_MANAGEMENT = 1 + 5
} NWDPTrainingEnum, N_FAR *pNWDPTrainingEnum;

typedef struct {
	NWDPNameOrOid identifier;
	NWDPStateSeverityEnum severity;
	NWDPTrainingEnum trainingLevel;
	NWDPObjectIdentifier objectClassOid;
	NWDPObjectIdentification objectIdentification;
	nuint32 time;
	pNWDPNameOrOid messageOptionPtr;
} NWDPPrinterStateReason, N_FAR *pNWDPPrinterStateReason;


typedef enum {
	NWDP_PERSISTENCE_PERMANENT = 0,
	NWDP_PERSISTENCE_VOLATILE = 1
} NWDPPersistenceEnum, N_FAR *pNWDPPersistenceEnum;

typedef enum {
	NWDP_ADDR_ITEM_USER = 0,
	NWDP_ADDR_ITEM_SERVER = 1,
	NWDP_ADDR_ITEM_VOLUME = 2,
	NWDP_ADDR_ITEM_ORG_UNIT = 3,
	NWDP_ADDR_ITEM_ORG = 4,
	NWDP_ADDR_ITEM_GROUP = 5,
	NWDP_ADDR_ITEM_DN = 6,
	NWDP_ADDR_ITEM_USR_OR_CONTAINER = 7,
	NWDP_ADDR_ITEM_CASE_EXACT_STR = 8,
	NWDP_ADDR_ITEM_CASE_IGNORE_STR = 9,
	NWDP_ADDR_ITEM_NUMERIC_STR = 10,
	NWDP_ADDR_ITEM_DOS_FILENAME = 11,
	NWDP_ADDR_ITEM_PHONE_NUMBER = 12,
	NWDP_ADDR_ITEM_BOOLEAN = 13,
	NWDP_ADDR_ITEM_INTEGER = 14,
	NWDP_ADDR_ITEM_NET_ADDRESS = 15,
	NWDP_ADDR_ITEM_CHOICE = 16,
	NWDP_ADDR_ITEM_GROUPWISE_USER = 17
} NWDPAddressItemTypeEnum, N_FAR *pNWDPAddressItemTypeEnum;

typedef struct {
	NWDPAddressItemTypeEnum designator;
	union {
		NWDPQualifiedName userName;
		NWDPQualifiedName serverName;
		NWDPQualifiedName volumeName;
		NWDPQualifiedName orgUnitName;
		NWDPQualifiedName orgName;
		NWDPQualifiedName groupName;
		NWDPQualifiedName dN;
		NWDPQualifiedName userOrContainerName;
		NWDPText ceStr;
		NWDPText ciStr;
		NWDPText numStr;
		NWDPText filename;
		NWDPText phoneNumber;
		nbool bFlag;
		nint32 integer;
		NWDPNetAddress netAddress;
		NWDPText choice;
		NWDPText gwUser;
	} u;
} NWDPAddressItem, N_FAR *pNWDPAddressItem;

typedef struct {
	nuint itemCount;
	pNWDPAddressItem itemPtr;
} NWDPNotifyDeliveryAddr, N_FAR *pNWDPNotifyDeliveryAddr;

typedef enum {
	NWDP_EVENT_OBJ_OP_NONE = 0,
	NWDP_EVENT_OBJ_OP_ADD_FLAG = 1,
	NWDP_EVENT_OBJ_OP_DELETE_FLAG = 2,
	NWDP_EVENT_OBJ_OP_DELETE_OBJECT = 3
} NWDPEventObjectOperation, N_FAR *pNWDPEventObjectOperation;

typedef enum {
	NWDP_EVENT_OBJ_TYPE_OBJECT = 0,
	NWDP_EVENT_OBJ_TYPE_FILTER = 1,
	NWDP_EVENT_OBJ_TYPE_DETAIL = 2
} NWDPEventObjectTypeEnum, N_FAR *pNWDPEventObjectTypeEnum;

typedef struct {
	NWDPEventObjectTypeEnum designator;
	union {
		NWDPObjectIdentifier filterClassOid;
		NWDPObjectIdentifierSet eventOidSet;
	} u;
} NWDPEventObjectItem, N_FAR *pNWDPEventObjectItem;

typedef struct {
	nuint32 eventType;
	NWDPObjectIdentifier containingClassOid;
	NWDPObjectIdentification containingObjectId;
	NWDPEventObjectOperation opCode;
	NWDPEventObjectItem item;
} NWDPEventObject, N_FAR *pNWDPEventObject;

typedef struct {
	nuint itemCount;
	pNWDPEventObject itemPtr;
} NWDPEventObjectSet, N_FAR *pNWDPEventObjectSet;

typedef struct {
	nuint32 profileId;
	NWDPPersistenceEnum persistence;
	NWDPQualifiedName consumerName;
	NWDPOctetString supplierId;
	nint32 languageId;
	NWDPNameOrOid methodId;
	NWDPNotifyDeliveryAddr deliveryAddress;
	NWDPEventObjectSet eventObjectSet;
	NWDPQualifiedName accountOption;
} NWDPEventHandlingProfile, N_FAR *pNWDPEventHandlingProfile;

typedef struct {
	nuint32 profileId;
	NWDPPersistenceEnum persistence;
	NWDPQualifiedName consumerName;
	NWDPOctetString supplierId;
	nint32 languageId;
	NWDPNameOrOid methodId;
	NWDPNotifyDeliveryAddr deliveryAddress;
	NWDPEventObjectSet eventObjectSet;
	NWDPQualifiedName accountOption;
	NWDPObjectIdentifierSet notifyAttributes;
	nuint32 notifyTimeInterval;
	nuint32 notifySequenceNumber;
	nuint32 notifyLeaseExpirationTime;
	NWDPText notifyPrinterUri;
} NWDPEventHandlingProfile2, N_FAR *pNWDPEventHandlingProfile2;

typedef enum {
	NWDP_LIST_CHOICE_ID = 0,
	NWDP_LIST_CHOICE_FILTER = 1
} NWDPListProfilesChoiceEnum, N_FAR *pNWDPListProfilesChoiceEnum;

typedef enum {
	NWDP_LIST_RESULT_COMPLETE = 0,
	NWDP_LIST_RESULT_NO_EVENT_OBJS = 1,
	NWDP_LIST_RESULT_PROFILE_IDS = 2
} NWDPListProfilesResultEnum, N_FAR *pNWDPListProfilesResultEnum;

typedef struct {
	NWDPQualifiedName consumerNameOption;
	pNWDPNameOrOid methodIdOptionPtr;
	NWDPIntegerOption languageId;
} NWDPListProfilesFilter, N_FAR *pNWDPListProfilesFilter;

typedef struct {
	NWDPListProfilesChoiceEnum designator;
	union {
		NWDPCardinalSeq profileIdSeq;
		NWDPListProfilesFilter filter;
	} u;
} NWDPListProfilesChoice, N_FAR *pNWDPListProfilesChoice;

typedef enum {
	NWDP_LIST_EVENT_PROFILES_SPEC = 0,
	NWDP_LIST_EVENT_PROFILES_CONT = 1
} NWDPListProfilesTypeEnum, N_FAR *pNWDPListProfilesTypeEnum;

typedef struct {
	NWDPQualifiedName supplierAlias;
	NWDPListProfilesChoice choice;
	NWDPListProfilesResultEnum detail;
	NWDPIntegerOption countOption;
} NWDPListEventProfilesSpec, N_FAR *pNWDPListEventProfilesSpec;

typedef struct {
	NWDPOctetString context;
	nbool abort;
} NWDPListEventProfilesCont, N_FAR *pNWDPListEventProfilesCont;

typedef struct {
	NWDPListProfilesTypeEnum designator;
	union {
		NWDPListEventProfilesSpec specification;
		NWDPListEventProfilesCont continuation;
	} u;
} NWDPListProfilesType, N_FAR *pNWDPListProfilesType;

typedef struct {
	nuint itemCount;
	pNWDPEventHandlingProfile itemPtr;
} NWDPProfileResultSet, N_FAR *pNWDPProfileResultSet;

typedef struct {
	NWDPObjectIdentifier deliveryMethodOption;
	NWDPNameOrOid resultsSetCommentOption;
	pNWDPDeliveryAddress deliveryAddressOptionPtr;
	nint32 jobCopies;
	NWDPPrtContainedObjectId outputBinOption;
} NWDPResultsProfile, N_FAR *pNWDPResultsProfile;

typedef struct {
	NWDPAVTEnum designator;
	union {
		nint32 deltaTime;
		nuint32 time;
		nint32 integer;
		nuint32 cardinal;
		nint32 positiveInteger;
		nint32 percent;
	} u;
} NWDPCriterionThreshold, N_FAR *pNWDPCriterionThreshold;

typedef struct {
	NWDPObjectIdentifier type;
	NWDPCriterionThreshold threshold;
} NWDPCriteria, N_FAR *pNWDPCriteria;

typedef struct {
	NWDPObjectIdentifier organizationId;
	nuint32 level;
} NWDPJobLevel, N_FAR *pNWDPJobLevel;

typedef struct {
	NWDPObjectIdentifier organizationId;
	NWDPCardinalSeq categories;
} NWDPJobCategories, N_FAR *pNWDPJobCategories;

typedef struct {
	NWDPAVTEnum designator;
	union {
		NWDPObjectIdentifier identifier;
		NWDPNameOrOid nameOrOid;
	} u;
} NWDPIgnoredAttributeValue, N_FAR *pNWDPIgnoredAttributeValue;

typedef struct {
	nuint itemCount;
	pNWDPIgnoredAttributeValue itemPtr;
} NWDPIgnoredAttrValueSet, N_FAR *pNWDPIgnoredAttrValueSet;

typedef struct {
	nuint32 documentNumberOption;
	NWDPObjectIdentifier attributeId;
	NWDPIgnoredAttrValueSet values;
} NWDPIgnoredAttribute, N_FAR *pNWDPIgnoredAttribute;

typedef enum {
	NWDP_RESOURCE_NAME_OR_OID = 0,
	NWDP_RESOURCE_TEXT = 1
} NWDPResourceEnum, N_FAR *pNWDPResourceEnum;

typedef struct {
	NWDPResourceEnum designator;
	union {
		NWDPNameOrOid nameOrOid;
		NWDPText name;
	} u;
} NWDPResource, N_FAR *pNWDPResource;

typedef struct {
	NWDPNameOrOid original;
	NWDPNameOrOid substitution;
} NWDPMediumSubstitution, N_FAR *pNWDPMediumSubstitution;

typedef struct {
	NWDPText original;
	NWDPText substitution;
} NWDPFontSubstitution, N_FAR *pNWDPFontSubstitution;

typedef enum {
	NWDP_RESOURCE_CONTXT_NAM_OR_OID = 0,
	NWDP_RESOURCE_CONTEXT_TEXT = 1
} NWDPResourceContextEnum, N_FAR *pNWDPResourceContextEnum;

typedef struct {
	NWDPResourceContextEnum designator;
	union {
		NWDPNameOrOid objectName;
		NWDPText pathName;
	} u;
} NWDPResourceContext, N_FAR *pNWDPResourceContext;

typedef struct {
	nuint itemCount;
	pNWDPResourceContext itemPtr;
} NWDPResourceContextSeq, N_FAR *pNWDPResourceContextSeq;

typedef enum {
	NWDP_PAGE_ID_NOMINAL_NUMBER = 0,
	NWDP_PAGE_ID_ALPHANUMERIC = 1,
	NWDP_PAGE_ID_TAG = 2
} NWDPPageIdentifierEnum, N_FAR *pNWDPPageIdentifierEnum;

typedef struct {
	NWDPPageIdentifierEnum designator;
	union {
		nint32 nominalPageNumber;
		NWDPText alphanumericPageNumber;
		NWDPNameOrOid pageTag;
	} u;
} NWDPPageIdentifier, N_FAR *pNWDPPageIdentifier;

typedef struct {
	nint32 flag;
	NWDPPageIdentifier value;
} NWDPPageIdentifierOption, N_FAR *pNWDPPageIdentifierOption;

typedef struct {
	NWDPPageIdentifierOption beginningPage;
	NWDPPageIdentifierOption endingPage;
} NWDPPageSelect, N_FAR *pNWDPPageSelect;

typedef struct {
	nuint itemCount;
	pNWDPPageSelect itemPtr;
} NWDPPageSelectSequence, N_FAR *pNWDPPageSelectSequence;

typedef enum {
	NWDP_DOC_CONTENT_INCLUDED = 0,
	NWDP_DOC_CONTENT_REFERENCED = 1
} NWDPDocumentContentEnum, N_FAR *pNWDPDocumentContentEnum;

typedef struct {
	NWDPDocumentContentEnum designator;
	union {
		NWDPOctetString includedDocument;
		NWDPDistinguishedNameString referencedDocument;
	} u;
} NWDPDocumentContent, N_FAR *pNWDPDocumentContent;

typedef enum {
	NWDP_PAGE_SIZE_ID = 0,
	NWDP_PAGE_SIZE_DIMENSIONS = 1
} NWDPPageSizeEnum, N_FAR *pNWDPPageSizeEnum;

typedef struct {
	NWDPPageSizeEnum designator;
	union {
		NWDPObjectIdentifier pageSizeId;
		NWDPXYRealDimensions pageDimensions;
	} u;
} NWDPPageSize, N_FAR *pNWDPPageSize;

typedef enum {
	NWDP_DIR_TO_RIGHT_TO_BOTTOM = 0,
	NWDP_DIR_TO_LEFT_TO_BOTTOM = 1,
	NWDP_DIR_BIDIRECT_TO_BOTTOM = 2,
	NWDP_DIR_TO_RIGHT_TO_TOP = 3,
	NWDP_DIR_TO_LEFT_TO_TOP = 4,
	NWDP_DIR_BIDIRECT_TO_TOP = 5,
	NWDP_DIR_TO_BOTTOM_TO_RIGHT = 6,
	NWDP_DIR_TO_BOTTOM_TO_LEFT = 7,
	NWDP_DIR_TO_TOP_TO_LEFT = 8,
	NWDP_DIR_TO_TOP_TO_RIGHT = 9
} NWDPPresentatnDirectionEnum, N_FAR *pNWDPPresentatnDirectionEnum;

typedef enum {
	NWDP_PAGE_ORDER_UNKNOWN = 0,
	NWDP_PAGE_ORDER_FIRST_TO_LAST = 1,
	NWDP_PAGE_ORDER_LAST_TO_FIRST = 2
} NWDPPageOrderTypeEnum, N_FAR *pNWDPPageOrderTypeEnum;

typedef enum {
	NWDP_PAGE_ID_TYPE_NUMERIC = 0,
	NWDP_PAGE_ID_TYPE_ALPHANUMERIC = 1,
	NWDP_PAGE_ID_TYPE_TAG = 2
} NWDPPageIdTypeEnum, N_FAR *pNWDPPageIdTypeEnum;

typedef enum {
	NWDP_MEDIA_SELECT_ALL_PAGES = 0,
	NWDP_MEDIA_SELECT_SELECTD_PAGES = 1
} NWDPMediaSelectEnum, N_FAR *pNWDPMediaSelectEnum;

typedef struct {
	NWDPPageSelect pageRange;
	NWDPNameOrOid mediumId;
} NWDPMediaSelect, N_FAR *pNWDPMediaSelect;

typedef struct {
	nuint itemCount;
	pNWDPMediaSelect itemPtr;
} NWDPMediaSelectSeq, N_FAR *pNWDPMediaSelectSeq;

typedef struct {
	NWDPMediaSelectEnum designator;
	union {
		NWDPNameOrOid mediumForAllPages;
		NWDPMediaSelectSeq mediumForSelectedPagesSeq;
	} u;
} NWDPPageMediaSelect, N_FAR *pNWDPPageMediaSelect;

typedef enum {
	NWDP_MEDIUM_SIZE_DISCRETE = 0,
	NWDP_MEDIUM_SIZE_CONTINUOUS = 1
} NWDPMediumSizeEnum, N_FAR *pNWDPMediumSizeEnum;

typedef struct {
	NWDPPageSize pageSize;
	nbool longEdgeFeeds;
	NWDPArea assuredReproductionArea;
} NWDPMediumDiscreteSizes, N_FAR *pNWDPMediumDiscreteSizes;

typedef struct {
	NWDPRealRange sizeRangeAcrossFeedDirection;
	nreal64 sizeIncrementAcrossFeedDir;
	NWDPRealRange sizeRangeInFeedDirection;
	nreal64 sizeIncrementInFeedDir;
	nbool longEdgeFeeds;
	NWDPArea GenericAssuredReproductionArea;
} NWDPMediumContinuousSizes, N_FAR *pNWDPMediumContinuousSizes;

typedef struct {
	NWDPMediumSizeEnum designator;
	union {
		NWDPMediumDiscreteSizes discreteSizes;
		NWDPMediumContinuousSizes continuousSizes;
	} u;
} NWDPMediumSize, N_FAR *pNWDPMediumSize;

typedef struct {
	pNWDPNameOrOid inputTrayOptionPtr;
	NWDPMediumSize mediumSize;
} NWDPMediumSourceSize, N_FAR *pNWDPMediumSourceSize;

typedef struct {
	NWDPNameOrOid inputTray;
	NWDPNameOrOid medium;
} NWDPInputTrayMedium, N_FAR *pNWDPInputTrayMedium;

typedef enum {
	NWDP_PAGE_FACE_UNKNOWN = 0,
	NWDP_PAGE_FACE_UP = 1,
	NWDP_PAGE_FACE_DOWN = 2
} NWDPPageOrientationEnum, N_FAR *pNWDPPageOrientationEnum;

typedef struct {
	NWDPPageOrderTypeEnum stackingOrder;
	NWDPPageOrientationEnum orientation;
} NWDPOutputBinChar, N_FAR *pNWDPOutputBinChar;

typedef struct {
	nuint itemCount;
	pNWDPOutputBinChar itemPtr;
} NWDPOutBinsCharacteristics, N_FAR *pNWDPOutBinsCharacteristics;

typedef struct {
	NWDPObjectIdentifier organizationId;
	nint32 minimum;
	nint32 maximum;
} NWDPLevelRange, N_FAR *pNWDPLevelRange;

typedef struct {
	NWDPObjectIdentifier organizationId;
	NWDPCardinalSeq categoryMinimum;
	NWDPCardinalSeq categoryMaximum;
} NWDPCategorySet, N_FAR *pNWDPCategorySet;

typedef enum {
	NWDP_NUMBERS_UP_CARDINAL = 0,
	NWDP_NUMBERS_UP_NAME_OR_OID = 1,
	NWDP_NUMBERS_UP_CARDINAL_RANGE = 2
} NWDPNumbersUpEnum, N_FAR *pNWDPNumbersUpEnum;

typedef struct {
	NWDPNumbersUpEnum designator;
	union {
		nuint32 cardinal;
		NWDPNameOrOid nameOrOid;
		NWDPCardinalRange cardinalRange;
	} u;
} NWDPNumbersUpSupported, N_FAR *pNWDPNumbersUpSupported;

typedef enum {
	NWDP_IMP_ATTR_PLEX = 0,
	NWDP_IMP_ATTR_NUMBER_UP = 1,
	NWDP_IMP_ATTR_X_IMAGE_SHIFT = 2,
	NWDP_IMP_ATTR_Y_IMAGE_SHIFT = 3,
	NWDP_IMP_ATTR_LOG_PAGE_ORIGIN = 4,
	NWDP_IMP_ATTR_LOG_PAGE_X_OFFSET = 5,
	NWDP_IMP_ATTR_LOG_PAGE_Y_OFFSET = 6,
	NWDP_IMP_ATTR_LOG_PAGE_SCALING = 7
} NWDPImpositionAttrEnum, N_FAR *pNWDPImpositionAttrEnum;

typedef struct {
	NWDPImpositionAttrEnum designator;
	union {
		struct {
			nuint itemCount;
			pnint8 itemPtr;
		} plex;
		nint32 numberUp;
		nreal64 xImageShift;
		nreal64 yImageShift;
		struct {
			nuint itemCount;
			pnint8 itemPtr;
		} logPageOrigin;
		nreal64 logPageXOffset;
		nreal64 logPageYOffset;
		nreal64 logPageScaling;
	} u;
} NWDPImpositionAttr, N_FAR *pNWDPImpositionAttr;

typedef enum {
	NWDP_IMPOSITION_OBJECT = 0,
	NWDP_IMPOSITION_ATTR = 1
} NWDPImpositionEnum, N_FAR *pNWDPImpositionEnum;

typedef struct {
	NWDPImpositionEnum designator;
	union {
		NWDPNameOrOid impositionObject;
		NWDPImpositionAttr impositionAttr;
	} u;
} NWDPImposition, N_FAR *pNWDPImposition;

typedef struct {
	NWDPObjectIdentifier colorantSetClass;
	NWDPNameOrOidSet colorantSpecSeq;
} NWDPColorantSetDescr, N_FAR *pNWDPColorantSetDescr;

typedef enum {
	NWDP_COLORANT_SET_NAME = 0,
	NWDP_COLORANT_SET_DESCR = 1
} NWDPColorantEnum, N_FAR *pNWDPColorantEnum;

typedef struct {
	NWDPColorantEnum designator;
	union {
		NWDPNameOrOid colorantSetName;
		NWDPColorantSetDescr colorantSetDescr;
	} u;
} NWDPColorantSet, N_FAR *pNWDPColorantSet;

typedef struct {
	NWDPText keyName;
	NWDPText mfrName;
	NWDPText infFileName;
	nint32 osType;
} NWDPResourcePrinterId, N_FAR *pNWDPResourcePrinterId;

typedef struct {
	NWDPAVTEnum designator;
	union {
		NWDPText text;
		NWDPText descriptiveName;
		NWDPText descriptor;
		NWDPNameOrOid message;
		NWDPNameOrOid errorMessage;
		NWDPText simpleName;
		NWDPDistinguishedNameString distinguishedNameString;
		NWDPDistinguishedNameStrSeq distinguishedNameStringSeq;
		nint32 deltaTime;
		nuint32 time;
		nint32 integer;
		NWDPIntegerSeq integerSeq;
		nuint32 cardinal;
		NWDPCardinalSeq cardinalSeq;
		nint32 positiveInteger;
		NWDPIntegerRange integerRange;
		NWDPCardinalRange cardinalRange;
		nint32 maximumInteger;
		nint32 minimumInteger;
		nreal64 real;
		NWDPRealSet realSeq;
		nreal64 nonNegativeReal;
		NWDPRealRange realRange;
		NWDPRealRange nonNegativeRealRange;
		nbool boolean;
		nint32 percent;
		NWDPObjectIdentifier identifier;
		NWDPObjectIdentifierSet objectIdentifierSeq;
		NWDPNameOrOid nameOrOid;
		NWDPNameOrOidSet nameOrOidSeq;
		NWDPText distinguishedName;
		NWDPRDNSequence rdnSequence;
		NWDPRealizationEnum realization;
		NWDPXYRealDimensions mediumDimensions;
		pNWDPDimension dimensionPtr;
		pNWDPXYDimensions xyDimensionsPtr;
		pNWDPLocations locationsPtr;
		pNWDPArea areaPtr;
		NWDPAreaSeq areaSeq;
		NWDPEdge edge;
		NWDPText fontReference;
		NWDPCardinalOrOid cardinalOrOid;
		NWDPOidCardinalMap oidCardinalMap;
		NWDPCardinalOrNameOrOid cardinalOrNameOrOid;
		NWDPPositiveIntegerOrOid positiveIntegerOrOid;
		pNWDPEventHandlingProfile eventHandlingProfilePtr;
		NWDPOctetString octetString;
		nint32 jobPriority;
		NWDPText locale;
		pNWDPDeliveryAddrForMethod deliveryMethodAddressPtr;
		pNWDPObjectIdentification objectIdentificationPtr;
		pNWDPResultsProfile resultsProfilePtr;
		NWDPCriteria criteria;
		NWDPOctetString jobPassword;
		NWDPJobLevel jobLevel;
		NWDPJobCategories jobCategories;
		NWDPOctetString printCheckpoint;
		NWDPIgnoredAttribute ignoredAttribute;
		NWDPResource resource;
		pNWDPMediumSubstitution mediumSubstitutionPtr;
		NWDPFontSubstitution fontSubstitution;
		NWDPResourceContextSeq resourceContextSeq;
		nint32 sides;
		NWDPPageSelectSequence pageSelectSeq;
		NWDPPageMediaSelect pageMediaSelect;
		NWDPDocumentContent documentContent;
		NWDPPageSize pageSize;
		NWDPPresentatnDirectionEnum presentationDirection;
		NWDPPageOrderTypeEnum pageOrderType;
		NWDPDistinguishedNameString fileReference;
		pNWDPMediumSourceSize mediumSourceSizePtr;
		pNWDPInputTrayMedium inputTrayMediumPtr;
		NWDPOutBinsCharacteristics outBinsCharacteristics;
		NWDPPageIdTypeEnum pageIdType;
		NWDPLevelRange levelRange;
		pNWDPCategorySet categorySetPtr;
		NWDPNumbersUpSupported numbersUpSupported;
		NWDPPrtContainedObjectId prtContainedObjectId;
		pNWDPPrtConfigObjectId prtConfigObjectIdPtr;
		NWDPTypedName typedName;
		NWDPNetAddress netAddress;
		NWDPXYDimensionsValue xyDimensionsValue;
		NWDPNameOrOidDimensionMap nameOrOidDimensionMap;
		pNWDPPrinterStateReason printerStateReasonPtr;
		nint32 enumeration;
		NWDPQualifiedName qualifiedName;
		NWDPQualifiedNameSet qualifiedNameSet;
		NWDPColorantSet colorantSet;
		pNWDPResourcePrinterId resourcePrinterIdPtr;
		NWDPEventObjectId eventObjectId;
		pNWDPQualifiedNameMap qualifiedNameMapPtr;
		NWDPText filePath;
		NWDPText uniformResourceIdentifier;
		NWDPCardinalOrEnumOrTime cardinalOrEnumOrTime;
		NWDPPrtContainedObjectIdSet prtContainedObjectIdSet;
		NWDPOctetStringPair octetStringPair;
		NWDPOctetStringIntegerPair octetStringIntegerPair;
		NWDPText extendedResourceIdentifier;
		pNWDPEventHandlingProfile2 eventHandlingProfile2Ptr;
	} u;
} NWDPAttributeValue, N_FAR *pNWDPAttributeValue;

typedef struct {
	nuint itemCount;
	pNWDPAttributeValue itemPtr;
} NWDPAttributeValueSet, N_FAR *pNWDPAttributeValueSet;

typedef enum {
	NWDP_MODIFY_OP_NULL = 0,
	NWDP_MODIFY_OP_REPLACE = 1,
	NWDP_MODIFY_OP_ADD_VALUES = 2,
	NWDP_MODIFY_OP_REMOVE_VALUES = 3,
	NWDP_MODIFY_OP_SET_TO_DEFAULT = 4,
	NWDP_MODIFY_OP_REMOVE_ATTRIBUTE = 5
} NWDPModifyOperatorEnum, N_FAR *pNWDPModifyOperatorEnum;
/* Note: For NDPS, all attribute sets are qualified */


typedef struct {
	NWDPObjectIdentifier attributeId;
	NWDPAttributeValueSet valueSet;
	nuint32 qualifier;
} NWDPAttribute, N_FAR *pNWDPAttribute;

typedef struct {
	nuint itemCount;
	pNWDPAttribute itemPtr;
} NWDPAttributeSet, N_FAR *pNWDPAttributeSet;

typedef enum {
	NWDP_MATCH_EXACT = 0,
	NWDP_MATCH_CASE_INSENSITIVE = 1,
	NWDP_MATCH_SAME_LETTER = 2,
	NWDP_MATCH_APPROXIMATE = 3
} NWDPSubstrMatchCriteriaEnum, N_FAR *pNWDPSubstrMatchCriteriaEnum;

typedef struct {
	NWDPObjectIdentifier attributeId;
	NWDPSubstrMatchCriteriaEnum matchCriteria;
	NWDPAttributeValue initialValueOption;
	NWDPAttributeValueSet anyValueSetOption;
	NWDPAttributeValue finalValueOption;
} NWDPSubStrings, N_FAR *pNWDPSubStrings;
typedef enum {
	NWDP_PROBLEM_TYPE_STANDARD = 0,
	NWDP_PROBLEM_TYPE_EXTENDED = 1
} NWDPProblemTypeEnum, N_FAR *pNWDPProblemTypeEnum;

typedef enum {
	NWDP_ACCESS_WRONG_OBJECT_CLASS = 0,
	NWDP_ACCESS_LACK_ACCESS_RIGHTS = 1,
	NWDP_ACCESS_CANT_INTERRUPT_JOB = 2,
	NWDP_ACCESS_WRONG_OBJECT_STATE = 3,
	NWDP_ACCESS_CLIENT_NOT_BOUND = 4,
	NWDP_ACCESS_NOT_AVAILABLE = 5,
	NWDP_ACCESS_NOTIF_SVC_NOT_CONN = 6,
	NWDP_ACCESS_PDS_NOT_CONNECTED = 7
} NWDPAccessProblemEnum, N_FAR *pNWDPAccessProblemEnum;

typedef struct {
	NWDPProblemTypeEnum designator;
	union {
		NWDPAccessProblemEnum standardProblem;
		NWDPObjectIdentifier extendedProblem;
	} u;
} NWDPAccessProblem, N_FAR *pNWDPAccessProblem;

typedef struct {
	NWDPAccessProblem problem;
	NWDPObjectIdentification objectIdentification;
	pNWDPNameOrOid messageOptionPtr;
} NWDPAccessError, N_FAR *pNWDPAccessError;

typedef enum {
	NWDP_ATTR_INVALID_SYNTAX = 0,
	NWDP_ATTR_UNDEFINED_TYPE = 1,
	NWDP_ATTR_WRONG_MATCHING = 2,
	NWDP_ATTR_CONSTRAINT_VIOLATED = 3,
	NWDP_ATTR_UNSUPPORTED_TYPE = 4,
	NWDP_ATTR_ILLEGAL_MODIFICATION = 5,
	NWDP_ATTR_CONSIST_W_OTHER_ATTR = 6,
	NWDP_ATTR_UNDEFINED_ATTR_VALUE = 7,
	NWDP_ATTR_UNSUPPORTED_VALUE = 8,
	NWDP_ATTR_INVALID_NONCOMPUL_MOD = 9,
	NWDP_ATTR_PER_JOB_INADMISSIBLE = 10,
	NWDP_ATTR_NOT_MULTI_VALUED = 11,
	NWDP_ATTR_MANDATORY_OMITTED = 12,
	NWDP_ATTR_ILLEGAL_FOR_CLASS = 13
} NWDPAttributeProblemEnum, N_FAR *pNWDPAttributeProblemEnum;

typedef struct {
	NWDPProblemTypeEnum designator;
	union {
		NWDPAttributeProblemEnum standardProblem;
		NWDPObjectIdentifier extendedProblem;
	} u;
} NWDPAttributeProblem, N_FAR *pNWDPAttributeProblem;

typedef struct {
	NWDPAttributeProblem problem;
	NWDPAttribute attribute;
	pNWDPNameOrOid messageOptionPtr;
} NWDPAttributeProblemItem, N_FAR *pNWDPAttributeProblemItem;

typedef struct {
	struct {
		nuint itemCount;
		pNWDPAttributeProblemItem itemPtr;
	} problems;
	NWDPObjectIdentification objectIdentification;
} NWDPAttributeError, N_FAR *pNWDPAttributeError;

typedef enum {
	NWDP_DOC_ACCESS_NOT_AVAILABLE = 0,
	NWDP_DOC_ACCESS_FIDELI_TIME_EXP = 1,
	NWDP_DOC_ACCESS_DENIED = 2,
	NWDP_DOC_ACCESS_UNKNOWN_DOC = 3,
	NWDP_DOC_ACCESS_NO_DOCS_IN_JOB = 4
} NWDPDocAccessProblemEnum, N_FAR *pNWDPDocAccessProblemEnum;

typedef struct {
	NWDPProblemTypeEnum designator;
	union {
		NWDPDocAccessProblemEnum standardProblem;
		NWDPObjectIdentifier extendedProblem;
	} u;
} NWDPDocAccessProblem, N_FAR *pNWDPDocAccessProblem;

typedef struct {
	NWDPDocAccessProblem problem;
	NWDPObjectIdentification objectIdentification;
	pNWDPNameOrOid messageOptionPtr;
} NWDPDocAccessError, N_FAR *pNWDPDocAccessError;

typedef enum {
	NWDP_PRINTER_ERROR = 0,
	NWDP_PRINTER_NEEDS_ATTENTION = 1,
	NWDP_PRINTER_NEEDS_KEY_OPERATOR = 2
} NWDPPrinterProblemEnum, N_FAR *pNWDPPrinterProblemEnum;

typedef struct {
	NWDPProblemTypeEnum designator;
	union {
		NWDPPrinterProblemEnum standardProblem;
		NWDPObjectIdentifier extendedProblem;
	} u;
} NWDPPrinterProblem, N_FAR *pNWDPPrinterProblem;

typedef struct {
	NWDPPrinterProblem problem;
	NWDPObjectIdentification objectIdentification;
	pNWDPNameOrOid messageOptionPtr;
} NWDPPrinterError, N_FAR *pNWDPPrinterError;

typedef enum {
	NWDP_SECURITY_AUTHENTICATION = 0,
	NWDP_SECURITY_CREDENTIALS = 1,
	NWDP_SECURITY_RIGHTS = 2,
	NWDP_SECURITY_INVALID_PAC = 3
} NWDPSecurityProblemEnum, N_FAR *pNWDPSecurityProblemEnum;

typedef struct {
	NWDPProblemTypeEnum designator;
	union {
		NWDPSecurityProblemEnum standardProblem;
		NWDPObjectIdentifier extendedProblem;
	} u;
} NWDPSecurityProblem, N_FAR *pNWDPSecurityProblem;

typedef struct {
	NWDPSecurityProblem problem;
	pNWDPNameOrOid messageOptionPtr;
} NWDPSecurityError, N_FAR *pNWDPSecurityError;

typedef enum {
	NWDP_SELECTION_INVALID_ID = 0,
	NWDP_SELECTION_UNKNOWN_ID = 1,
	NWDP_SELECTION_OBJECT_EXISTS = 2,
	NWDP_SELECTION_ID_CHANGED = 3
} NWDPSelectionProblemEnum, N_FAR *pNWDPSelectionProblemEnum;

typedef struct {
	NWDPProblemTypeEnum designator;
	union {
		NWDPSelectionProblemEnum standardProblem;
		NWDPObjectIdentifier extendedProblem;
	} u;
} NWDPSelectionProblem, N_FAR *pNWDPSelectionProblem;

typedef struct {
	NWDPSelectionProblem problem;
	NWDPObjectIdentification objectIdentification;
	pNWDPAttribute attributeOptionPtr;
	pNWDPNameOrOid messageOptionPtr;
} NWDPSelectionError, N_FAR *pNWDPSelectionError;

typedef enum {
	NWDP_SERVICE_SERVER_BUSY = 0,
	NWDP_SERVICE_SERVER_UNAVAILABLE = 1,
	NWDP_SERVICE_OPERATION_COMPLEX = 2,
	NWDP_SERVICE_RESOURCE_LIMIT = 3,
	NWDP_SERVICE_UNCLASS_SERVER_ERR = 4,
	NWDP_SERVICE_TOO_MANY_ITEM_LIST = 5,
	NWDP_SERVICE_RESOURCE_NOT_AVAIL = 6,
	NWDP_SERVICE_CANCEL_DOC_SUPPORT = 7,
	NWDP_SERVICE_MODIFY_DOC_SUPPORT = 8,
	NWDP_SERVICE_MULTI_DOC_SUPPORT = 9,
	NWDP_SERVICE_PARM_VAL_SUPPORT = 10,
	NWDP_SERVICE_INVALID_CHECKPOINT = 11,
	NWDP_SERVICE_CONTINUATN_CONTEXT = 12,
	NWDP_SERVICE_PAUSE_LIMIT_EXCEED = 13,
	NWDP_SERVICE_UNSUPPORTED_OP = 14,
	NWDP_SERVICE_NOTIF_SERVICE_ERR = 15,
	NWDP_SERVICE_ACCTG_SERVICE_ERR = 16
} NWDPServiceProblemEnum, N_FAR *pNWDPServiceProblemEnum;

typedef struct {
	NWDPProblemTypeEnum designator;
	union {
		NWDPServiceProblemEnum standardProblem;
		NWDPObjectIdentifier extendedProblem;
	} u;
} NWDPServiceProblem, N_FAR *pNWDPServiceProblem;

typedef struct {
	NWDPServiceProblem problem;
	NWDPObjectIdentification objectIdentification;
	pNWDPAttribute attributeOptionPtr;
	nuint32 libError;
	nuint32 otherError;
	nuint32 otherError2;
	pNWDPNameOrOid messageOptionPtr;
} NWDPServiceError, N_FAR *pNWDPServiceError;

typedef enum {
	NWDP_UPDATE_NO_MODS_ALLOWED = 0,
	NWDP_UPDATE_INSUFFICIENT_RIGHTS = 1,
	NWDP_UPDATE_PREV_OP_INCOMPLETE = 2,
	NWDP_UPDATE_CANCEL_NOT_POSSIBLE = 3
} NWDPUpdateProblemEnum, N_FAR *pNWDPUpdateProblemEnum;

typedef struct {
	NWDPProblemTypeEnum designator;
	union {
		NWDPUpdateProblemEnum standardProblem;
		NWDPObjectIdentifier extendedProblem;
	} u;
} NWDPUpdateProblem, N_FAR *pNWDPUpdateProblem;

typedef struct {
	NWDPUpdateProblem problem;
	NWDPObjectIdentification objectIdentification;
	pNWDPNameOrOid messageOptionPtr;
} NWDPUpdateError, N_FAR *pNWDPUpdateError;

typedef enum {
	NWDP_ERROR_TYPE_SECURITY = 0,
	NWDP_ERROR_TYPE_SERVICE = 1,
	NWDP_ERROR_TYPE_ACCESS = 2,
	NWDP_ERROR_TYPE_PRINTER = 3,
	NWDP_ERROR_TYPE_SELECTION = 4,
	NWDP_ERROR_TYPE_DOCUMENT_ACCESS = 5,
	NWDP_ERROR_TYPE_ATTRIBUTE = 6,
	NWDP_ERROR_TYPE_UPDATE = 7
} NWDPErrorTypeEnum, N_FAR *pNWDPErrorTypeEnum;

typedef struct {
	NWDPErrorTypeEnum designator;
	union {
		NWDPSecurityError securityError;
		struct {
			nuint itemCount;
			pNWDPServiceError itemPtr;
		} serviceErrorSeq;
		struct {
			nuint itemCount;
			pNWDPAccessError itemPtr;
		} accessErrorSeq;
		NWDPPrinterError printerError;
		struct {
			nuint itemCount;
			pNWDPSelectionError itemPtr;
		} selectionErrorSeq;
		NWDPDocAccessError docAccessError;
		NWDPAttributeError attributeError;
		NWDPUpdateError updateError;
	} u;
} NWDPErrorReturn, N_FAR *pNWDPErrorReturn;

typedef struct {
	NWDPText name;
	struct {
		nuint itemCount;
		pnint8 itemPtr;
	} password;
} NWDPCreds, N_FAR *pNWDPCreds;

typedef struct {
	pnstr serverNamePtr;
	nuint16 connection;
} NWDPNdpsCred0, N_FAR *pNWDPNdpsCred0;

typedef struct {
	pnstr serverNamePtr;
	nuint16 connection;
	NWDPText clientName;
} NWDPNdpsCred1, N_FAR *pNWDPNdpsCred1;

typedef struct {
	pnstr serverNamePtr;
	nuint16 connection;
	NWDPText clientName;
	NWDPNetAddress clientAddress;
} NWDPNdpsCred2, N_FAR *pNWDPNdpsCred2;

typedef enum {
	NWDP_CREDENTIALS_SIMPLE = 0,
	NWDP_CREDENTIALS_CERTIFIED = 1,
	NWDP_CREDENTIALS_NDPS_0 = 2,
	NWDP_CREDENTIALS_NDPS_1 = 3,
	NWDP_CREDENTIALS_NDPS_2 = 4
} NWDPCredentialsEnum, N_FAR *pNWDPCredentialsEnum;

typedef struct {
	NWDPCredentialsEnum designator;
	union {
		NWDPCreds simple;
		struct {
			nuint itemCount;
			pnint8 itemPtr;
		} certified;
		NWDPNdpsCred0 ndpsCred0;
		NWDPNdpsCred1 ndpsCred1;
		NWDPNdpsCred2 ndpsCred2;
	} u;
} NWDPCredentials, N_FAR *pNWDPCredentials;


#include <npackoff.h>


/* Structure Definitions */

/* Global Variable Declarations */

/* Function Prototype Declarations */



#else /* NWDP_ATT_H is defined. */

   #if defined(N_DEBUG)
      #error nwdp_att.h has already been included.
   #endif /* N_DEBUG */

#endif /* NWDP_ATT_H */

